#pragma once

#include "syntax/CodeBlock.hpp"
#include "syntax/DivBlock.hpp"
#include "syntax/EOF.hpp"
#include "syntax/EnumerateBlock.hpp"
#include "syntax/ExecutableCodeBlock.hpp"
#include "syntax/Header.hpp"
#include "syntax/HorizontalLine.hpp"
#include "syntax/InlineCodeBlock.hpp"
#include "syntax/InlineImage.hpp"
#include "syntax/InlineItalic.hpp"
#include "syntax/InlineMath.hpp"
#include "syntax/InlineOverline.hpp"
#include "syntax/InlineStrong.hpp"
#include "syntax/InlineUrl.hpp"
#include "syntax/Item.hpp"
#include "syntax/Judge.hpp"
#include "syntax/ListBlock.hpp"
#include "syntax/LoadLib.hpp"
#include "syntax/Markdown.hpp"
#include "syntax/MathBlock.hpp"
#include "syntax/NewLine.hpp"
#include "syntax/Quote.hpp"
#include "syntax/RawText.hpp"
#include "syntax/Table.hpp"

#include "syntax/InlineFootnoteReference.hpp"
#include "syntax/FootnoteDefinition.hpp"